
N = North = 0
E = East = 1
S = South = 2
W = West = 3
U = Up = 4
D = Down = 5

DISPLAY_NONE  = 0
DISPLAY_LABEL = 1
DISPLAY_GLUES = 2
DISPLAY_ALL   = 3

DirAbbrevArray = ("N", "E", "S", "W", "U", "D")
DirNameArray = ("NORTH", "EAST", "SOUTH", "WEST", "UP", "DOWN")


class Glue:
    """A representation of a glue on the side of a tile, including strength and label"""

    def __init__(self, strength = 0, label = ""):
        self.strength = strength
        self.label = label
        self.hash = None

    def __eq__(self, other):
        if (isinstance(other, Glue)):
            return (self.strength == other.strength) and (self.label == other.label)
        return False
    
    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return str(self.strength) + ":" + self.label

    def Copy(self):
        return Glue(self.strength, self.label)

    def Contains(self, string):
        return (self.label.find(string) > -1)
    
    def Equals(self, other):
        if (self.strength == other.strength) and (self.label == other.label):
            return True
        return False

    def Print(self, d):
        print(DirNameArray[d] + "BIND " + str(self.strength))
        print(DirNameArray[d] + "LABEL " + self.label)

    def Output(self, d):
        ret  = DirNameArray[d] + "BIND " + str(self.strength) + "\n"
        ret += DirNameArray[d] + "LABEL " + self.label + "\n"
        return ret
    
    def Hash(self):
        if self.hash is None:
            self.hash = self.label + str(self.strength)
        return self.hash

#----------------------------------------------------------------------------

class TileType:
    def __init__(self, name, width=100, height=100, label=None):
        self.name = name
        self.label = label
        self.color = "White"
        self.colorVector = (255, 255, 255)
        self.textColor = "Black"
        self.textColorVector = (0, 0, 0)
        self.width = width
        self.height = height
        self.dims = 2
        self.sides = {N:Glue(),E:Glue(),S:Glue(),W:Glue()}
        self.fullString = None

    def __hash__(self):
        sidesStr = ""
        for s in self.sides:
            sidesStr = sidesStr + '$' + str(self.sides[s])
        return hash(sidesStr)
    
    def __eq__(self, other):
        if (isinstance(other, type(self))):
            return (self.__hash__() == other.__hash__())
        return False

    # function to parse the color string into a color vector
    def ParseColor(self):
        if self.color.lower().startswith("rgba"):
            try:
                self.colorVector = eval(self.color[4:])
            except:
                self.colorVector = (255, 255, 255)
        else:
            col = wx.TheColourDatabase.Find(self.color)
            if col.IsOk():
                self.colorVector = (col.Red(), col.Green(), col.Blue())
            else:
                if not self.color.lower().startswith("rgb"):
                    self.colorVector = (255, 255, 255)
                else:
                    channels = self.color[4:-1].split(",")
                    try:
                        r = int(channels[0].strip())
                        g = int(channels[1].strip())
                        b = int(channels[2].strip())
                        self.colorVector = (r, g, b)
                    except:
                        #print "Color Error on Tile: {}".format(self.name)#DEBUG
                        self.colorVector = (255, 255, 255)

        if self.textColor.lower().startswith("rgba"):
            try:
                self.textColorVector = eval(self.textColor[4:])
            except:
                self.textColorVector = (255, 255, 255)
        else:
            col = wx.TheColourDatabase.Find(self.textColor)
            if col.IsOk():
                self.textColorVector = (col.Red(), col.Green(), col.Blue())
            else:
                if not self.textColor.lower().startswith("rgb"):
                    self.textColorVector = (255, 255, 255)
                else:
                    channels = self.textColor[4:-1].split(",")
                    try:
                        r = int(channels[0].strip())
                        g = int(channels[1].strip())
                        b = int(channels[2].strip())
                        self.textColorVector = (r, g, b)
                    except:
                        #print "Color Error on Tile: {}".format(self.name)#DEBUG
                        self.textColorVector = (0, 0, 0)
    
    def Copy(self):
        t = TileType(self.name)
        t.label = self.label
        t.color = self.color
        t.colorVector = self.colorVector
        t.textColorVector = self.textColorVector
        t.width = self.width
        t.height = self.height
        t.dims = self.dims
        t.textColor = self.textColor
        t.sides[N] = self.sides[N].Copy()
        t.sides[E] = self.sides[E].Copy()
        t.sides[S] = self.sides[S].Copy()
        t.sides[W] = self.sides[W].Copy()
        t.fullString = self.fullString
        return t
    
    def SetEqual(self, other):
        self.name = other.name
        self.label = other.label
        self.color = other.color
        self.colorVector = other.colorVector
        self.textColorVector = other.textColorVector
        self.width = other.width
        self.height = other.height
        self.dims = other.dims
        self.textColor = other.textColor
        self.sides[N] = other.sides[N].Copy()
        self.sides[E] = other.sides[E].Copy()
        self.sides[S] = other.sides[S].Copy()
        self.sides[W] = other.sides[W].Copy()
        self.fullString = other.fullString
        
    def Print(self):
        if self.IsEmpty():
            return
        if self.name == "":
            self.CreateName()
        print("TILENAME " + self.name)
        print("LABEL " + self.label)
        for i,d in self.sides.items():
            self.sides[i].Print(i)
        if len(self.color) > 0:
            print("TILECOLOR " + self.color)
        if len(self.textColor) > 0:
            print("TEXTCOLOR " + str(self.textColor))
        print("CREATE\n")

    def Output(self):
        if self.IsEmpty():
            return ""
        if self.name == "":
            self.CreateName()
        ret  = "TILENAME " + self.name + "\n"
        if not self.label is None:
            ret += "LABEL " + self.label + "\n"
        for i,d in self.sides.items():
            ret += self.sides[i].Output(i)
        if (len(self.color) > 0) and (not self.color == "White"):
            ret += "TILECOLOR " + self.color + "\n"
        if (len(self.textColor) > 0) and (not self.textColor == "Black"):
            ret += "TEXTCOLOR " + str(self.textColor) + "\n"
        ret += "CREATE\n"
        return ret
    
    def HasMatchingGlues(self, other):
        if not len(self.sides) == len(other.sides):
            return False
        for side in self.sides:
            if not side in other.sides:
                return False
            elif self.sides[side].Equals(other.sides[side]) == False:
                return False
        return True
    
    def IsEmpty(self):
        for i in range(self.dims*2):
            if not self.sides[i].strength == 0:
                return False
        return True

    def GetName(self):
        if len(self.name) == 0:
            self.CreateName()
        return self.name

    def CreateName(self):
        self.name = ""
        for i in range(self.dims*2):
            if len(self.name) > 0:
                self.name = self.name + ","
            self.name = self.name + "[" + DirAbbrevArray[i] + "]" + str(self.sides[i].strength) + ":" + self.sides[i].label

    def __str__(self):
        return self.GetName()


###################################################

CELLSIZE = 180

#----------------------------------------------------------

class PyTASTileType(TileType):
    """A representation of a tile type for use in the ISU TAS simulator.
       Every tile must have a unique name and a unique set of 4 glues. If
       a name is not explicitly provided, one will be formed automatically
       as a concatenation of the glues values of each side."""
    
    def __init__(self, name="", width=CELLSIZE, height=CELLSIZE, is3D=False, label=""):
        TileType.__init__(self, name, width, height, label)
        if is3D == False:
            self.dims = 2
            self.sides = {N:Glue(),E:Glue(),S:Glue(),W:Glue()}
        else:
            self.dims = 3
            self.sides = {N:Glue(),E:Glue(),S:Glue(),W:Glue(),U:Glue(),D:Glue()}
           
    def SetDimensions(self, dims):
        if self.dims == dims:
            return
        elif dims == 3:
            self.dims = 3
            self.sides[U] = Glue()
            self.sides[D] = Glue()
        elif dims == 2:
            self.dims = 2
            savedSides = self.sides
            self.sides = {N:savedSides[N], E:savedSides[E], S:savedSides[S], W:savedSides[W]}
        else:
            print("ERROR: invalid number of tile type dimensions: " + str(dims))

    def Copy(self):
        if self.dims == 2:
            t = PyTASTileType()
        else:
            t = TileType(True)
        t.sides[N] = self.sides[N].Copy()
        t.sides[E] = self.sides[E].Copy()
        t.sides[S] = self.sides[S].Copy()
        t.sides[W] = self.sides[W].Copy()
        if self.dims == 3:
            t.sides[U] = self.sides[U].Copy()
            t.sides[D] = self.sides[D].Copy()
        t.name = self.name
        t.label = self.label
        t.color = self.color
        t.textColor = self.textColor
        t.colorVector = self.colorVector
        t.textColorVector = self.textColorVector
        for imgNum in range(len(self.images)):
            if self.images[imgNum] is None:
                t.images[imgNum] = None
            else:
                t.images[imgNum] = self.images[imgNum].GetSubBitmap(wx.Rect(0, 0, self.images[imgNum].GetWidth(), self.images[imgNum].GetHeight()))
        return t
    
    def SetEqual(self, other):
        TileType.SetEqual(self, other)
        for imgNum in range(len(self.images)):
            if other.images[imgNum] is None:
                self.images[imgNum] = None
            else:
                self.images[imgNum] = other.images[imgNum].GetSubBitmap(wx.Rect(0, 0, other.images[imgNum].GetWidth(), other.images[imgNum].GetHeight()))

    def SetInputSides(self, inputSet):
        for k in inputSet:
            self.sides[k] = Glue(inputSet[k].strength, inputSet[k].label)


    
