#####################################################
# This file must define each of the following:
#   1. h_glue_suffix
#   2. v_glue_suffix
#   3. io_conventions
#   4. macrotile_base (TODO: this will probably need to be a dictionary to handle more generic designs)
#   5. macrotile_addition_map


#####################################################
# Define the horizontal and vertical glue suffixes to determine which has the 'regular' domains and which
# has the complements
h_glue_suffix = '*'
v_glue_suffix = ''

#####################################################
# Define the io_conventions function that takes as input a tuple specifying direction name and either 'in' or 'out'
# and returns a function that will return True for macrotile locations that should contain slats
def io_conventions(args):
    if args in [(N,'in'), (E,'in'), (S,'out'), (W,'out')]:
        return even
    elif args in [(N,'out'), (E,'out'), (S,'in'), (W,'in')]:
        return odd
    else:
        print(f'ERROR: Invalid args {args} to io_conventions.')
        quit()

#####################################################
# Define the base of the macrotile, to be used for the macrotiles created from all tiles
macrotile_base = []

#####################################################
## Define dictionary that maps an input or output glue to the necessary slats and domains for the macrotile
# No a-t-g or mismatches
NE_SW_coop_map_3x3 = {
    'name': 'NE_SW',
    (N, 'in', 1):  [{'coords': [(1,2),(2,2)], 'inputs': [((1,2),N), ((1,1),N)]},],

    (E, 'in', 1):  [{'coords': [(1,2),(2,2)], 'inputs': [((2,2),N)]}],

    (S, 'out', 1): [{'coords': [(1,1),(1,2)], 'active_slats': (io_conventions,(N,'out'))},
                    {'coords': [(0,1),(1,1)]},
                    {'coords': [(0,0),(0,1)]},
                    {'coords': [(0,0),(1,0)]},
                    {'coords': [(1,-2),(1,0)], 'active_slats': (io_conventions,(S,'out')), 'outputs': [((1,-1),S), ((1,-2),S)], 'color': 'orange'}],
    (S,'out',2): [{'coords': [(1,1),(1,2)], 'active_slats': (io_conventions,(N,'out'))},
                  {'coords': [(0,1),(1,1)]},
                  {'coords': [(0,0),(0,1)]},
                  {'coords': [(0,0),(1,0)]},
                  {'coords': [(1,-1),(1,0)], 'outputs': [((1,-1),S)], 'color': 'orange'}],

    (W, 'out', 1): [{'coords': [(1,1),(1,2)], 'active_slats': (io_conventions,(N,'out'))},
                    {'coords': [(0,1),(1,1)]},
                    {'coords': [(-1,1),(0,1)], 'color': 'orange'},
                    {'coords': [(-1,0),(-1,2)], 'active_slats': (io_conventions,(S,'out')), 'outputs': [((-1,2),S),((-1,0),S)], 'color': 'orange'}],
    (W, 'out', 2): [{'coords': [(1,1),(1,2)], 'active_slats': (io_conventions,(N,'out'))},
                    {'coords': [(0,1),(1,1)]},
                    {'coords': [(0,1),(-1,1)], 'outputs': [((-1,1),W)], 'color': 'orange'}],
    }

NW_SE_coop_map_3x3 = {
    'name': 'NW_SE',
    (N,'in',1): [{'coords': [(0,2),(1,2)], 'inputs': [((1,2),N), ((1,1),N)]}],
   
    (E,'out',1): [{'coords': [(1,1),(1,2)], 'active_slats': (io_conventions,(N,'out'))},
                  {'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(3,1)], 'color': 'orange'},
                  {'coords': [(3,2),(3,0)], 'active_slats': (io_conventions, (N,'out')), 'outputs': [((3,2),N), ((3,0),N)], 'color': 'orange'}
                  ],

    (E,'out',2): [{'coords': [(1,1),(1,2)], 'active_slats': (io_conventions,(N,'out'))},
                  {'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(3,1)], 'outputs': [((3,1),E)], 'color': 'orange'}],
    
    (S,'out',1): [{'coords': [(1,1),(1,2)], 'active_slats': (io_conventions,(N,'out'))},
                  {'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0), (1,0)]},
                  {'coords': [(1,0),(1,-2)], 'active_slats': (io_conventions, (S,'out')), 'outputs': [((1,-1),S), ((1,-2),S)], 'color': 'orange'}],
    (S,'out',2): [{'coords': [(1,1),(1,2)], 'active_slats': (io_conventions,(N,'out'))},
                  {'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0), (1,0)]},
                  {'coords':[(1,0),(1,-1)], 'outputs': [((1,-1),S)], 'color': 'orange'}],
    
    (W,'in',1): [{'coords': [(0,2),(1,2)], 'inputs': [((0,2),S)]}],

}
S2_N2_out_W_coop_map_3x3 = {
    'name': 'S2_N2',
    (N,'out',1): [{'coords': [(1,0),(0,0)]},
                  {'coords': [(0,0), (0,2)]},
                  {'coords': [(0,2),(1,2)]},
                  {'coords': [(1,2),(1,4)], 'active_slats': (io_conventions, (N,'out')), 'outputs': [((1,3),N), ((1,4),N)], 'color': 'orange'}],
    (N,'out',2):  [{'coords': [(1,0),(0,0)]},
                  {'coords': [(0,0), (0,2)]},
                  {'coords': [(0,2),(1,2)]},
                  {'coords': [(1,2),(1,3)], 'outputs': [((1,3), N)], 'color': 'orange'}],  

    (E,'out',1): [{'coords': [(1,0),(2,0)]},
                  {'coords': [(2,0),(2,1)]},
                  {'coords': [(2,1),(3,1)], 'color': 'orange'},
                  {'coords': [(3,2),(3,0)], 'active_slats': (io_conventions, (N,'out')), 'outputs': [((3,2),N), ((3,0),N)], 'color': 'orange'}],
    (E,'out',2): [{'coords': [(1,0),(2,0)]},
                  {'coords': [(2,0),(2,1)]},
                  {'coords': [(2,1),(3,1)], 'outputs': [((3,1),E)], 'color': 'orange'}],

    (S,'in',2): [{'coords': [(1,0)],'orientation': Orient.H, 'inputs': [((1,0),S)]}],

    (S,'out',1): [{'coords': [(1,0)], 'orientation': Orient.H},
                  {'coords': [(1,0),(1,-2)], 'outputs': [((1,-1),S), ((1,-2),S)], 'color': 'orange'}],

    (S,'out',2): [{'coords': [(1,0)], 'orientation': Orient.H},
                  {'coords': [(1,0),(1,-1)], 'outputs': [((1,-1),S)], 'color': 'orange'}],
    
    (W,'out',1): [{'coords': [(1,0),(0,0)]},
                  {'coords': [(0,0),(0,1)]},
                  {'coords': [(0,1),(-1,1)], 'color': 'orange'},
                  {'coords': [(-1,0),(-1,2)], 'active_slats': (io_conventions,(S,'out')), 'outputs': [((-1,2),S),((-1,0),S)], 'color': 'orange'}],
    (W,'out',2): [{'coords': [(1,0),(0,0)]},
                  {'coords': [(0,0),(0,1)]},
                  {'coords': [(0,1),(-1,1)], 'outputs': [((-1,1),W)], 'color': 'orange'}]
}

S2_N2_out_E_coop_map_3x3 = {
    'name': 'S2_N2',
    (N,'out',1): [{'coords': [(1,0),(2,0)]},
                  {'coords': [(2,0), (2,2)]},
                  {'coords': [(2,2),(1,2)]},
                  {'coords': [(1,2),(1,4)], 'active_slats': (io_conventions, (N,'out')), 'outputs': [((1,3),N), ((1,4),N)], 'color': 'orange'}],
    (N,'out',2):  [{'coords': [(1,0),(2,0)]},
                  {'coords': [(2,0), (2,2)]},
                  {'coords': [(2,2),(1,2)]},
                  {'coords': [(1,2),(1,3)], 'outputs': [((1,3), N)], 'color': 'orange'}],  

    (E,'out',1): [{'coords': [(1,0),(2,0)]},
                  {'coords': [(2,0),(2,1)]},
                  {'coords': [(2,1),(3,1)], 'color': 'orange'},
                  {'coords': [(3,2),(3,0)], 'active_slats': (io_conventions, (N,'out')), 'outputs': [((3,2),N), ((3,0),N)], 'color': 'orange'}],
    (E,'out',2): [{'coords': [(1,0),(2,0)]},
                  {'coords': [(2,0),(2,1)]},
                  {'coords': [(2,1),(3,1)], 'outputs': [((3,1),E)], 'color': 'orange'}],

    (S,'in',2): [{'coords': [(1,0)],'orientation': Orient.H, 'inputs': [((1,0),S)]}],

    (S,'out',1): [{'coords': [(1,0)], 'orientation': Orient.H},
                  {'coords': [(1,0),(1,-2)], 'outputs': [((1,-1),S), ((1,-2),S)], 'color': 'orange'}],

    (S,'out',2): [{'coords': [(1,0)], 'orientation': Orient.H},
                  {'coords': [(1,0),(1,-1)], 'outputs': [((1,-1),S)], 'color': 'orange'}],
    
    (W,'out',1): [{'coords': [(1,0),(0,0)]},
                  {'coords': [(0,0),(0,1)]},
                  {'coords': [(0,1),(-1,1)], 'color': 'orange'},
                  {'coords': [(-1,0),(-1,2)], 'active_slats': (io_conventions,(S,'out')), 'outputs': [((-1,2),S),((-1,0),S)], 'color': 'orange'}],
    (W,'out',2): [{'coords': [(1,0),(0,0)]},
                  {'coords': [(0,0),(0,1)]},
                  {'coords': [(0,1),(-1,1)], 'outputs': [((-1,1),W)], 'color': 'orange'}]
}

E2_W2_out_S_coop_map_3x3 = {
    'name': 'E2-W2',
    (N,'out',1): [{'coords': [(2,1),(2,2)]},
                  {'coords': [(2,2),(1,2)]},
                  {'coords': [(1,2),(1,4)], 'active_slats': (io_conventions, (N,'out')), 'outputs': [((1,3),N), ((1,4),N)], 'color': 'orange'}],
    (N,'out',2): [{'coords': [(2,1),(2,2)]},
                  {'coords': [(2,2),(1,2)]},
                  {'coords': [(1,2),(1,3)], 'outputs': [((1,3),N)], 'color': 'orange'}],

    (E,'in',2): [{'coords': [(2,1)], 'orientation': Orient.V, 'inputs': [((2,1),E)]}],

    (S,'out',1): [{'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(1,0)]},
                  {'coords': [(1,0),(1,-2)], 'active_slats': (io_conventions, (S,'out')), 'outputs': [((1,-1),S), ((1,-2),S)], 'color': 'orange'}],

    (S,'out',2): [{'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(1,0)]},
                  {'coords': [(1,0),(1,-1)], 'outputs': [((1,-1),S)], 'color': 'orange'}],
    
    (W,'out',1): [{'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(0,0)]},
                  {'coords': [(0,0),(0,1)]},
                  {'coords': [(0,1),(-1,1)], 'color': 'orange'},
                  {'coords': [(-1,0),(-1,2)], 'active_slats': (io_conventions,(S,'out')), 'outputs': [((-1,2),S),((-1,0),S)], 'color': 'orange'}],
    (W,'out',2): [{'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(0,0)]},
                  {'coords': [(0,0),(0,1)]},
                  {'coords': [(0,1),(-1,1)], 'outputs': [((-1,1),W)], 'color': 'orange'}]
}

E2_W2_out_N_coop_map_3x3 = {
    'name': 'E2-W2',
    (N,'out',1): [{'coords': [(2,1),(2,2)]},
                  {'coords': [(2,2),(1,2)]},
                  {'coords': [(1,2),(1,4)], 'active_slats': (io_conventions, (N,'out')), 'outputs': [((1,3),N), ((1,4),N)], 'color': 'orange'}],
    (N,'out',2): [{'coords': [(2,1),(2,2)]},
                  {'coords': [(2,2),(1,2)]},
                  {'coords': [(1,2),(1,3)], 'outputs': [((1,3),N)], 'color': 'orange'}],

    (E,'in',2): [{'coords': [(2,1)], 'orientation': Orient.V, 'inputs': [((2,1),E)]}],

    (S,'out',1): [{'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(1,0)]},
                  {'coords': [(1,0),(1,-2)], 'active_slats': (io_conventions, (S,'out')), 'outputs': [((1,-1),S), ((1,-2),S)], 'color': 'orange'}],

    (S,'out',2): [{'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(1,0)]},
                  {'coords': [(1,0),(1,-1)], 'outputs': [((1,-1),S)], 'color': 'orange'}],
    
    (W,'out',1): [{'coords': [(2,1),(2,2)]},
                  {'coords': [(2,2),(0,2)]},
                  {'coords': [(0,2),(0,1)]},
                  {'coords': [(0,1),(-1,1)], 'color': 'orange'},
                  {'coords': [(-1,0),(-1,2)], 'active_slats': (io_conventions,(S,'out')), 'outputs': [((-1,2),S),((-1,0),S)], 'color': 'orange'}],
    (W,'out',2): [{'coords': [(2,1),(2,2)]},
                  {'coords': [(2,2),(0,2)]},
                  {'coords': [(0,2),(0,1)]},
                  {'coords': [(0,1),(-1,1)], 'outputs': [((-1,1),W)], 'color': 'orange'}]
}


macrotile_mapping_list = [{'signatures': [{'in': {N:1, E:1}, 'out': {S:[0,1,2], W:[0,1,2]}}],
                           'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':NE_SW_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {S:1, W:1}, 'out': {N:[0,1,2], E:[0,1,2]}}],
                           'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':NE_SW_coop_map_3x3, 'rotation':180}]},
                          {'signatures': [{'in': {N:1, W:1}, 'out': {S:[0,1,2], E:[0,1,2]}}],
                           'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':NW_SE_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {S:1, E:1}, 'out': {N:[0,1,2], W:[0,1,2]}}],
                           'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':NW_SE_coop_map_3x3, 'rotation':180}]},
                          {'signatures': [{'in': {S:2}, 'out': {N:[0,1,2], E: 0, W: [0,1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':S2_N2_out_W_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {S:2}, 'out': {N:[0,1,2], E: [1,2], W: [0,1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':S2_N2_out_E_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {N:2}, 'out': {S:[0,1,2], E: 0, W: [0,1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':S2_N2_out_E_coop_map_3x3, 'rotation':180}]},
                          {'signatures': [{'in': {N:2}, 'out': {S:[0,1,2], E: [1,2], W: [0,1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':S2_N2_out_W_coop_map_3x3, 'rotation':180}]},
                          {'signatures': [{'in': {E:2}, 'out': {N:[0,1,2], S: 0, W: [0,1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':E2_W2_out_N_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {E:2}, 'out': {N:[0,1,2], S: [1,2], W: [0,1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':E2_W2_out_S_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {W:2}, 'out': {N:[0,1,2], E: [0,1,2], S: 0}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':E2_W2_out_S_coop_map_3x3, 'rotation':180}]},
                          {'signatures': [{'in': {W:2}, 'out': {N:[0,1,2], E: [0,1,2], S: [1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':E2_W2_out_N_coop_map_3x3, 'rotation':180}]},
                          {'signatures': [{'in': {}, 'out': {N:[0,1,2], E: [0,1,2], W: [0,1,2], S: [0,1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':S2_N2_out_E_coop_map_3x3, 'rotation':0}]}
                        ]
                          

