#####################################################
# This file must define each of the following:
#   1. h_glue_suffix
#   2. v_glue_suffix
#   3. io_conventions
#   4. macrotile_base (TODO: this will probably need to be a dictionary to handle more generic designs)
#   5. macrotile_addition_map


#####################################################
# Define the horizontal and vertical glue suffixes to determine which has the 'regular' domains and which
# has the complements
h_glue_suffix = '*'
v_glue_suffix = ''

#####################################################
# Define the io_conventions function that takes as input a tuple specifying direction name and either 'in' or 'out'
# and returns a function that will return True for macrotile locations that should contain slats
def io_conventions(args):
    if args in [(N,'in'), (E,'in'), (S,'out'), (W,'out')]:
        return even
    elif args in [(N,'out'), (E,'out'), (S,'in'), (W,'in')]:
        return odd
    else:
        print(f'ERROR: Invalid args {args} to io_conventions.')
        quit()

#####################################################
# Define the base of the macrotile, to be used for the macrotiles created from all tiles
macrotile_base = []

#####################################################
## Define dictionary that maps an input or output glue to the necessary slats and domains for the macrotile
# Standard Systems + Across The Gap
NE_coop_map_3x3 = {
    'name': 'NE_coop',
    (N, 'in', 1):  [{'coords': [(1,1), (2,1)], 'inputs': [((1,1), N)]}],

    (E, 'in', 1):  [{'coords': [(1,1),(2,1)], 'inputs': [((2,1),N)]}],

    (S, 'out', 1): [{'coords': [(1,1),(0,1)]},
                    {'coords': [(0,1),(0,0)]},
                    {'coords': [(0,0),(1,0)], 'active_slats': (io_conventions, (E,'out'))},
                    {'coords': [(1,0), (1,-2)], 'inputs': [((1,0),E)], 'outputs': [((1,-2), S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'},],
    (S, 'out', 2): [{'coords': [(1,1),(0,1)]},
                    {'coords': [(0,1),(0,0)]},
                    {'coords': [(0,0),(1,0)], 'active_slats': (io_conventions, (E,'out'))},
                    {'coords': [(1,0),(1,-2)], 'inputs': [((1,0),E)], 'outputs': [((1,-2), S)], 'color': 'orange'}],

    (W, 'out', 1): [{'coords': [(1,1),(0,1)]},
                    {'coords': [(0,1),(0,0)]},
                    {'coords': [(0,0),(-1,0)], 'color': 'orange'},
                    {'coords': [(-1,0),(-2,0)], 'outputs': [((-2,0), W)], 'active_slats': (io_conventions, (W,'out')), 'color': 'orange'},
                    {'coords': [(-1,0),(-1,1)], 'outputs': [((-1,1), S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'},
                    {'coords': [(-1,1),(-1,2)], 'outputs': [((-1,2),S)], 'active_slats': (io_conventions, (S,'out')), 'marker': 'ATG', 'color': 'orange'}],
    (W,'out', 2): [{'coords': [(1,1),(0,1)]},
                    {'coords': [(0,1),(0,0)]},
                    {'coords': [(0,0),(-1,0)], 'outputs': [((-1,0),W)], 'color': 'orange'}]
}

NW_coop_map_3x3 = {
    'name': 'NW_coop',
    (N,'in',1): [{'coords': [(0,1), (1,1)], 'inputs': [((1,1),N)]}],

    (E,'out',1): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1), (2,0)]},
                  {'coords': [(2,0),(3,0)]},
                  {'coords': [(3,0),(4,0)], 'outputs': [((4,0),E)], 'active_slats': (io_conventions, (E,'out')), 'color': 'orange'},
                  {'coords': [(3,0),(3,1)], 'outputs': [((3,1),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'},
                  {'coords': [(3,1),(3,2)], 'outputs': [((3,2),N)], 'active_slats': (io_conventions, (N,'out')), 'marker': 'ATG', 'color': 'orange'}],
    (E,'out',2): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1), (2,0)]},
                  {'coords': [(2,0),(3,0)], 'outputs': [((3,0),E)], 'color': 'orange'}],
    
    (S,'out',1): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1), (2,0)]},
                  {'coords': [(2,0),(1,0)], 'active_slats': (io_conventions, (W,'out'))},
                  {'coords': [(1,0),(1,-2)], 'inputs': [((1,0), W)], 'outputs': [((1,-2), S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'}],
    (S,'out',2): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1), (2,0)]},
                  {'coords': [(2,0),(1,0)], 'active_slats': (io_conventions, (W,'out'))},
                  {'coords': [(1,0),(1,-2)], 'inputs': [((1,0), W)], 'outputs': [((1,-2), S)], 'color': 'orange'}],

    (W,'in',1): [{'coords': [(0,1),(1,1)], 'inputs': [((0,1),S)]}],
}


SE_coop_map_3x3 = {
    'name': 'SE_coop',
    (N,'out',1): [{'coords': [(1,1),(0,1)]},
                  {'coords': [(0,1),(0,2)]},
                  {'coords': [(0,2),(1,2)]},
                  {'coords': [(1,2),(1,4)], 'outputs': [((1,4),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'}],
    (N,'out',2): [{'coords': [(1,1),(0,1)]},
                  {'coords': [(0,1),(0,2)]},
                  {'coords': [(0,2),(1,2)]},
                  {'coords': [(1,2),(1,4)], 'outputs': [((1,4),N)], 'color': 'orange'}],

    (E,'in',1): [{'coords': [(1,1),(2,1)], 'inputs': [((2,1), N)]}],
    
    (S,'in',1): [{'coords': [(1,1),(2,1)], 'inputs': [((1,1),S)]}],

    (W,'out',1): [{'coords': [(1,1),(0,1)]},
                  {'coords': [(0,1),(0,0)]},
                  {'coords': [(0,0),(-1,0)], 'color': 'orange'},
                  {'coords': [(-1,0),(-2,0)], 'outputs': [((-2,0), W)], 'active_slats': (io_conventions, (W,'out')), 'color': 'orange'},
                  {'coords': [(-1,0),(-1,1)], 'outputs': [((-1,1), S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'},
                  {'coords': [(-1,1),(-1,2)], 'outputs': [((-1,2),S)], 'active_slats': (io_conventions, (S,'out')), 'marker': 'ATG', 'color': 'orange'}],
    (W,'out',2): [{'coords': [(1,1),(0,1)]},
                  {'coords': [(0,1),(0,0)]},
                  {'coords': [(0,0),(-1,0)], 'outputs': [((-1,0),W)], 'color': 'orange'}]
}

SW_coop_map_3x3 = {
    'name': 'SW_coop',
    (N,'out',1): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,2)]},
                  {'coords': [(2,2),(1,2)]},
                  {'coords': [(1,2),(1,4)], 'outputs': [((1,4),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'}],
    (N,'out',2): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,2)]},
                  {'coords': [(2,2),(1,2)]},
                  {'coords': [(1,2),(1,4)], 'outputs': [((1,4),N)], 'color': 'orange'}],
    
    (E,'out',1): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(3,0)], 'color': 'orange'},
                  {'coords': [(3,0),(4,0)], 'outputs': [((4,0),E)], 'active_slats': (io_conventions, (E,'out')), 'color': 'orange'},
                  {'coords': [(3,0),(3,1)], 'outputs': [((3,1),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'},
                  {'coords': [(3,1),(3,2)], 'outputs': [((3,2),N)], 'active_slats': (io_conventions, (N,'out')), 'marker': 'ATG', 'color': 'orange'}],
    (E,'out',2): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(3,0)], 'outputs': [((3,0),E)], 'color': 'orange'}],


    (S,'in',1): [{'coords': [(0,1),(1,1)], 'inputs': [((1,1), S)]}],

    (W,'in', 1): [{'coords': [(0,1),(1,1)], 'inputs': [((0,1),S)]}]
}

N2_E2_S2_coop_map_3x3 = {
    'name': 'Strength_2_coop_N2_E2_S2',
    (N,'in',2): [{'coords': [(1,1)], 'orientation': Orient.H, 'inputs': [((1,1),N)]}],
    (N,'out',1): [{'coords': [(1,1), (0,1)]},
                  {'coords': [(0,1),(0,2)]},
                  {'coords': [(0,2),(1,2)]},
                  {'coords': [(1,2),(1,4)], 'outputs': [((1,4),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'}],
    (N,'out',2): [{'coords': [(1,1), (0,1)]},
                  {'coords': [(0,1),(0,2)]},
                  {'coords': [(0,2),(1,2)]},
                  {'coords': [(1,2),(1,4)], 'outputs': [((1,4),N)], 'color': 'orange'}],


    (E,'in',2): [{'coords': [(2,0),(2,1)], 'inputs': [((2,0),E)]},
                 {'coords': [(2,1),(1,1)]}],
    (E,'out',1): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(3,0)], 'color': 'orange'},
                  {'coords': [(3,0),(4,0)], 'outputs': [((4,0),E)], 'active_slats': (io_conventions, (E,'out')), 'color': 'orange'},
                  {'coords': [(3,0),(3,1)], 'outputs': [((3,1),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'},
                  {'coords': [(3,1),(3,2)], 'outputs': [((3,2),N)], 'active_slats': (io_conventions, (N,'out')), 'marker': 'ATG', 'color': 'orange'}],
    (E,'out',2): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(3,0)], 'outputs': [((3,0),E)], 'color': 'orange'}],
    
    (S,'in',2): [{'coords': [(1,1)], 'orientation': Orient.H, 'inputs': [((1,1),S)]}],
    (S,'out',1): [{'coords': [(1,1),(0,1)]},
                  {'coords': [(0,1),(0,0)]},
                  {'coords': [(0,0),(1,0)]},
                  {'coords': [(1,0),(1,-2)],  'outputs': [((1,-2), S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'}],
    (S,'out',2): [{'coords': [(1,1),(0,1)]},
                  {'coords': [(0,1),(0,0)]},
                  {'coords': [(0,0),(1,0)]},
                  {'coords': [(1,0),(1,-2)], 'outputs': [((1,-2), S)], 'color': 'orange'}],
    
    (W,'in',2): [{'coords': [(0,0),(0,1)], 'inputs': [((0,0),W)]},
                 {'coords': [(0,1),(1,1)]}],
    (W,'out',1): [{'coords': [(1,1),(0,1)]},
                  {'coords': [(0,1),(0,0)]},
                  {'coords': [(0,0),(-1,0)], 'color': 'orange'},
                  {'coords': [(-1,0),(-1,1)], 'outputs': [((-1,1), S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'},
                  {'coords': [(-1,1),(-1,2)], 'outputs': [((-1,2),S)], 'active_slats': (io_conventions, (S,'out')), 'marker': 'ATG', 'color': 'orange'},
                  {'coords': [(0,0),(-2,0)], 'outputs': [((-2,0), W)], 'active_slats': (io_conventions, (W,'out')), 'color': 'orange'}],
    (W,'out',2): [{'coords': [(1,1),(0,1)]},
                  {'coords': [(0,1),(0,0)]},
                  {'coords': [(0,0),(-1,0)], 'outputs': [((-1,0),W)], 'color': 'orange'}]

}

W2_coop_map_3x3 = {
    'name': 'Strength_2_coop_W2',
    (N,'out',1): [{'coords': [(1,1), (0,1)]},
                  {'coords': [(0,1),(0,2)]},
                  {'coords': [(0,2),(1,2)]},
                  {'coords': [(1,2),(1,4)], 'outputs': [((1,4),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'}],
    (N,'out',2): [{'coords': [(1,1), (0,1)]},
                  {'coords': [(0,1),(0,2)]},
                  {'coords': [(0,2),(1,2)]},
                  {'coords': [(1,2),(1,4)], 'outputs': [((1,4),N)], 'color': 'orange'}],


    (E,'out',1): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(3,0)], 'color': 'orange'},
                  {'coords': [(3,0),(4,0)], 'outputs': [((4,0),E)], 'active_slats': (io_conventions, (E,'out')), 'color': 'orange'},
                  {'coords': [(3,0),(3,1)], 'outputs': [((3,1),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'},
                  {'coords': [(3,1),(3,2)], 'outputs': [((3,2),N)], 'active_slats': (io_conventions, (N,'out')), 'marker': 'ATG', 'color': 'orange'}],
    (E,'out',2): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(3,0)], 'outputs': [((3,0),E)], 'color': 'orange'}],
    
    (S,'out',1): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(1,0)]},
                  {'coords': [(1,0),(1,-2)],  'outputs': [((1,-2), S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'}],
    (S,'out',2): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(1,0)]},
                  {'coords': [(1,0),(1,-2)], 'outputs': [((1,-2), S)], 'color': 'orange'}],
    
    (W,'in',2): [{'coords': [(0,0),(0,1)], 'inputs': [((0,0),W)]},
                 {'coords': [(0,1),(1,1)]}],
    (W,'out',1): [{'coords': [(1,1),(0,1)]},
                  {'coords': [(0,1),(0,0)]},
                  {'coords': [(0,0),(-1,0)], 'color': 'orange'},
                  {'coords': [(-1,0),(-1,1)], 'outputs': [((-1,1), S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'},
                  {'coords': [(-1,1),(-1,2)], 'outputs': [((-1,2),S)], 'active_slats': (io_conventions, (S,'out')), 'marker': 'ATG', 'color': 'orange'},
                  {'coords': [(0,0),(-2,0)], 'outputs': [((-2,0), W)], 'active_slats': (io_conventions, (W,'out')), 'color': 'orange'}],
    (W,'out',2): [{'coords': [(1,1),(0,1)]},
                  {'coords': [(0,1),(0,0)]},
                  {'coords': [(0,0),(-1,0)], 'outputs': [((-1,0),W)], 'color': 'orange'}]

}


EW_coop_map_3x3 = {
    'name': 'EW_coop',
    (N,'out',1): [{'coords': [(1,2),(1,4)], 'outputs': [((1,4),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'}],
    (N,'out',2): [{'coords': [(1,2),(1,4)], 'outputs': [((1,4),N)], 'color': 'orange'}],

    (E,'in',1): [{'coords': [(0,2),(2,2)], 'inputs': [((2,2),N)], 'marker': 'ATG'},
                 {'coords': [(1,0)], 'orientation': Orient.V, 'inputs': [((1,0),E)]}],
    
    (S,'out',1): [{'coords': [(1,0),(1,-2)], 'outputs': [((1,-2), S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'}],
    (S,'out',2): [{'coords': [(1,0),(1,-2)], 'outputs': [((1,-2), S)], 'color': 'orange'}],

    (W,'in',1): [{'coords': [(0,2),(2,2)], 'inputs': [((0,2),S)], 'marker': 'ATG'},
                 {'coords': [(1,0)], 'orientation': Orient.V, 'inputs': [((1,0),W)]}]
}

NS_coop_map_3x3 = {
    'name': 'NS_coop',

    (N,'in',1): [{'coords': [(1,1)], 'orientation': Orient.H, 'inputs': [((1,1),N)]}],

    (E,'out',1): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(3,0)], 'color': 'orange'},
                  {'coords': [(3,0),(4,0)], 'outputs': [((4,0),E)], 'active_slats': (io_conventions, (E,'out')), 'color': 'orange'},
                  {'coords': [(3,0),(3,1)], 'outputs': [((3,1),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'},
                  {'coords': [(3,1),(3,2)], 'outputs': [((3,2),N)], 'active_slats': (io_conventions, (N,'out')), 'marker': 'ATG', 'color': 'orange'}],
    (E,'out',2): [{'coords': [(1,1),(2,1)]},
                  {'coords': [(2,1),(2,0)]},
                  {'coords': [(2,0),(3,0)], 'outputs': [((3,0),E)], 'color': 'orange'}],

    (S,'in',1): [{'coords': [(1,1)], 'orientation': Orient.H, 'inputs': [((1,1),S)]}],

    (W,'out',1): [{'coords': [(1,1),(0,1)]},
                  {'coords': [(0,1),(0,0)]},
                  {'coords': [(0,0),(-1,0)], 'color': 'orange'},
                  {'coords': [(-1,0),(-2,0)], 'outputs': [((-2,0), E)], 'active_slats': (io_conventions, (E,'out')), 'color': 'orange'},
                  {'coords': [(-1,0),(-1,1)], 'outputs': [((-1,1), S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'},
                  {'coords': [(-1,1),(-1,2)], 'outputs': [((-1,2),S)], 'active_slats': (io_conventions, (S,'out')), 'marker': 'ATG', 'color': 'orange'}],
    (W,'out',2): [{'coords': [(1,1),(0,1)]},
                  {'coords': [(0,1),(0,0)]},
                  {'coords': [(0,0),(-1,0)], 'outputs': [((-1,0),W)], 'color': 'orange'}]

}

macrotile_mapping_list = [{'signatures': [{'in': {N:1, E:1}, 'out': {S:[0,1,2], W:[0,1,2]}}],
                           'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':NE_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {S:1, W:1}, 'out': {N:[0,1,2], E:[0,1,2]}}],
                           'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':SW_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {N:1, W:1}, 'out': {S:[0,1,2], E:[0,1,2]}}],
                           'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':NW_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {S:1, E:1}, 'out': {N:[0,1,2], W:[0,1,2]}}],
                           'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':SE_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {N:1, S:1}, 'out': {E:[0,1,2], W:[0,1,2]}}],
                           'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':NS_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {E:1, W:1}, 'out': {N:[0,1,2], S:[0,1,2]}}],
                           'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':EW_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {S:2}, 'out': {N:[0,1,2], E: [0,1,2], W: [0,1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':N2_E2_S2_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {N:2}, 'out': {S:[0,1,2], E: [0,1,2], W: [0,1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':N2_E2_S2_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {E:2}, 'out': {N:[0,1,2], S: [0,1,2], W: [0,1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':N2_E2_S2_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {W:2}, 'out': {N:[0,1,2], E: [0,1,2], S: [0,1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':W2_coop_map_3x3, 'rotation':0}]},
                          {'signatures': [{'in': {}, 'out': {N:[0,1,2], E: [0,1,2], S: [0,1,2], W:[0,1,2]}}],
                          'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':N2_E2_S2_coop_map_3x3, 'rotation':0}]},
                        ]
                          

#E -> N output
#W -> S output