#####################################################
# This file must define each of the following:
#   1. h_glue_suffix
#   2. v_glue_suffix
#   3. io_conventions
#   4. macrotile_base (TODO: this will probably need to be a dictionary to handle more generic designs)
#   5. macrotile_addition_map


#####################################################
# Define the horizontal and vertical glue suffixes to determine which has the 'regular' domains and which
# has the complements
h_glue_suffix = '*'
v_glue_suffix = ''

#####################################################
# Define the io_conventions function that takes as input a tuple specifying direction name and either 'in' or 'out'
# and returns a function that will return True for macrotile locations that should contain slats
def io_conventions(args):
    if args in [(N,'in'), (E,'in'), (S,'out'), (W,'out')]:
        return even
    elif args in [(N,'out'), (E,'out'), (S,'in'), (W,'in')]:
        return odd
    else:
        print(f'ERROR: Invalid args {args} to io_conventions.')
        quit()

#####################################################
# Define the base of the macrotile, to be used for the macrotiles created from all tiles
#Zig-zag Systems

empty_base = {}

doubled_outputs_seed_map = {'name': 'doubled_outputs_seed_map',
    (N, 'out', 1): [{'coords': [(0,1), (0,2)], 'outputs': [((0,2),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'}
                   ],
    (N, 'out', 2): [{'coords': [(1,1), (1,2)], 'outputs': [((1,2),N)], 'color': 'orange'}
                    ],
    (W,'out',2): [{'coords': [(0,1),(-1,1)], 'outputs': [((-1,1),W)], 'color': 'orange'}],
    
}
doubled_outputs_in_ES_out_NW = { 'name': 'doubled_outputs_in_ES_out_NW',
    (N,'out',1): [{'coords':[(0,0),(0,1)]},
                  {'coords':[(0,1),(0,2)], 'outputs': [((0,2),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'}],
    (N, 'out',2):  [{'coords':[(0,0),(0,1)]},
                  {'coords':[(0,1),(0,2)], 'outputs': [((0,2),N)], 'color': 'orange'}],
    (E,'in',1): [{'coords':[(1,0),(0,0)], 'inputs':[((1,0),N)]} 
                 ],
    (S,'in',1): [{'coords':[(1,0),(0,0)], 'inputs':[((1,0),S)]}],

    (S,'in',2): [{'coords':[(1,0),(0,0)], 'inputs':[((1,0),S)]}],

    (W,'out',1): [{'coords':[(0,0),(0,1)]},
                  {'coords':[(0,1),(-1,1)], 'color': 'orange'},
                  {'coords':[(-1,1),(-1,0)], 'outputs': [((-1,0),S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'}],
    
    (W,'out',2): [{'coords':[(0,0),(0,1)]},
                  {'coords':[(0,1),(-1,1)], 'outputs':[((-1,1),W)], 'color': 'orange'}]

}

doubled_outputs_in_S_out_NE = { 'name': 'doubled_outputs_in_S_out_NE',
    (N,'out',1): [{'coords':[(1,0),(1,1)]},
                  {'coords':[(1,1),(1,2)], 'outputs':[((1,2),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'}
                  ],
    (N, 'out', 2): [{'coords': [(1,0), (1,1)]},
                   {'coords': [(1,1), (1,2)], 'outputs': [((1,2),N)], 'color': 'orange'}
                    ],

    (E,'out',1): [{'coords':[(1,0),(1,1)]},
                  {'coords':[(1,1),(2,1)], 'color': 'orange'},
                  {'coords':[(2,1),(2,0)], 'outputs':[((2,0),S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'}],

    (E,'out',2): [{'coords':[(1,0),(1,1)]},
                  {'coords':[(1,1),(2,1)], 'outputs':[((2,1),E)], 'color': 'orange'}],
    
    
    (S,'in',2): [{'coords':[(0,0),(1,0)], 'inputs':[((0,0),S)]}]
}

double_outputs_in_SW_out_NE = { 'name': 'double_outputs_in_S1W1_out_N1E1_or_N1E2',
    (N,'out',1): [{'coords':[(1,0),(1,1)]},
                 {'coords':[(1,1),(1,2)], 'outputs':[((1,2),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'}],
    (N,'out',2): [{'coords':[(1,0),(1,2)], 'outputs':[((1,2),N)], 'color': 'orange'}],

    (E,'out',1): [{'coords':[(1,0),(1,1)]},
                 {'coords':[(1,1),(2,1)], 'color': 'orange'},
                 {'coords':[(2,1),(2,0)], 'outputs':[((2,0),S)], 'active_slats': (io_conventions, (S,'out')), 'color': 'orange'}],
    (E,'out',2): [{'coords':[(1,0),(1,1)]},
                 {'coords':[(1,1),(2,1)], 'outputs':[((2,1),E)], 'color': 'orange'}],

    (S,'in',1): [{'coords':[(0,0),(1,0)], 'inputs':[((0,0),S)]}],

    (W,'in',1): [{'coords':[(0,0),(1,0)], 'inputs':[((0,0),N)]}]
          
}

doubled_outputs_in_E_out_NW = { 'name': 'doubled_outputs_in_E_out_NW',
    (N,'out',1): [{'coords': [(1,0),(0,0)]},
                  {'coords': [(0,0),(0,1)]},
                  {'coords': [(0,1),(0,2)], 'outputs':[((0,2),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'}],
    (N,'out',2): [{'coords':[(1,0),(0,0)]},
                  {'coords':[(0,0),(0,2)], 'outputs':[((0,2),N)], 'color': 'orange'}
                  ],

    (E,'in',2): [{'coords': [(1,1),(1,0)], 'inputs': [((1,1), E)]}],

    (W,'out',2): [{'coords': [(1,0),(0,0)]},
                  {'coords': [(0,0),(0,1)]},
                  {'coords': [(0,1),(-1,1)], 'outputs': [((-1,1), W)], 'color': 'orange'}]

}

doubled_outputs_in_W_out_NE = { 'name': 'doubled_outputs_in_W_out_NE',
    (N,'out',1): [{'coords': [(0,0),(1,0)]},
                  {'coords': [(1,0),(1,1)]},
                  {'coords': [(1,1),(1,2)], 'outputs':[((1,2),N)], 'active_slats': (io_conventions, (N,'out')), 'color': 'orange'}],
    (N,'out',2): [{'coords':[(0,0),(1,0)]},
                  {'coords':[(1,0),(1,2)],'outputs':[((1,2),N)], 'color': 'orange'}],

    (E,'out',2): [{'coords': [(0,0),(1,0)]},
                  {'coords': [(1,0),(1,1)]},
                  {'coords': [(1,1),(2,1)], 'outputs': [((2,1),E)], 'color': 'orange'}],

    (W,'in',2): [{'coords': [(0,1),(0,0)], 'inputs': [((0,1),W)]}]

}

macrotile_mapping_list = [
                            {'signatures':[{'in': {E:1, S:1}, 'out': {N: [0,1,2], W:[0,1,2]}},
                                           {'in': {S:2}, 'out': {N:[0,1,2], W:[1,2]}},  
                                           {'in': {S:2}, 'out': {N:0, E:0, W:0}}],
                            'maps': [{'macrotile_base': empty_base, 'macrotile_additions_map':doubled_outputs_in_ES_out_NW, 'rotation':0}]},

                            {'signatures': [{'in': {S:1, W:1}, 'out': {N: [0,1,2], E:[0,1,2]}}],
                            'maps': [{'macrotile_base': empty_base, 'macrotile_additions_map': double_outputs_in_SW_out_NE, 'rotation':0}]},

                            {'signatures': [{'in': {E:2}, 'out': {N:[0,1,2], W:[0,2]}}],
                            'maps': [{'macrotile_base': empty_base, 'macrotile_additions_map': doubled_outputs_in_E_out_NW, 'rotation':0}]},

                            {'signatures': [{'in': {S:2}, 'out': {N:[0,1,2], E:[1,2]}}],
                            'maps': [{'macrotile_base': empty_base, 'macrotile_additions_map': doubled_outputs_in_S_out_NE, 'rotation':0}]},

                            {'signatures': [{'in': {W: 2}, 'out': {N: [0,1,2], E: [0,2]}}],
                            'maps': [{'macrotile_base': empty_base, 'macrotile_additions_map': doubled_outputs_in_W_out_NE, 'rotation':0}]},

                            {'signatures':[{'in': {S:0,E:0}, 'out': {N: [0,1,2], W:[0,2]}}],
                            'maps': [{'macrotile_base':empty_base, 'macrotile_additions_map':doubled_outputs_seed_map, 'rotation':0}]}
                           ]

                            

#macrotile_mapping_list = [{'signatures':[{'in': {S:2}, 'out': {W:[0,1,2], N:[0,1,2], E:[0,1,2]}},
#                                         {'in': {S:1, E:1}, 'out': {W:[0,1,2], N:[0,1,2]}},
#                                         {'in': {}, 'out': {S:[0,1,2], W:[0,1,2], N:[0,1,2], E:[0,1,2]}}],
#                           'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':doubled_outputs_seed_map, 'rotation':0}]},
#                          {'signatures':[{'in': {W:2}, 'out': {N:[0,1,2], E:[0,1,2], S:[0,1,2]}},
#                                         ],
#                           'maps':[{'macrotile_base':macrotile_base, 'macrotile_additions_map':doubled_outputs_macrotiles_map, 'rotation':90}]},
#                        #extra signature added to create different output interactions from tiles with 'in': {W:1, S:1}
#                          
#
#                          {'signatures': [{'in': {N: 2}, 'out': {E: [0, 1, 2], S: [0, 1, 2], W: [0, 1, 2]}},
#                                          {'in': {N: 1, W: 1}, 'out': {E: [0, 1, 2], S: [0, 1, 2]}}],
#                           'maps': [{'macrotile_base': macrotile_base,
#                                     'macrotile_additions_map': doubled_outputs_macrotiles_map, 'rotation': 180}]},
#                          {'signatures': [{'in': {E: 2}, 'out': {S: [0, 1, 2], W: [0, 1, 2], N: [0, 1, 2]}},
#                                          {'in': {E: 1, N: 1}, 'out': {S: [0, 1, 2], W: [0, 1, 2]}}],
#                           'maps': [{'macrotile_base': macrotile_base,
#                                     'macrotile_additions_map': doubled_outputs_macrotiles_map, 'rotation': 270}]}
#                          ]#