from slat_generator import *
from SlatSystem import *
import glob
import getopt
import sys


DISPLAY_ALL = 0
DISPLAY_GLUES = 1
DISPLAY_NO_GLUES = 2


def DisplayHelp():
    print("\nThis script takes as input the definition of a slat systems and assembly")
    print("and outputs it into SVG and/or PDF files.")
    print("\nUsage:")
    print("-h, --help\t\t\t\tDisplays this help message.")
    print("-x <xml file>,\t--xml=<xml file>\tXML file name of input file containing slat system.")
    print("\t\t\t\t\t Note that the xml_file parameter can also be a file name pattern to be matched.")
    print("-n <base name>,\t--name=<base name>\t[Optional] Base name of output files. If not specified,")
    print("\t\t\t\t\t the base name of the input XML file will be used.")
    print("-p, --pdf\t\t\t\t[Optional] If this option is used, the slats will be output to a PDF file.")
    print("-s, --svg\t\t\t\t[Optional] If this option is used, the slats will be output to an SVG file.")
    print("-d <dir>,\t--dir=<dir>\t\t[Optional] Directory for output files. If not specified,")
    print("\t\t\t\t\t the directory of the input file will be used.")
    print("-l <T/F>,\t--labels=<T/F>\t\t[Optional] Show lables, T for true, F for False (default T)")
    print("-f,\t\t--flip\t\t\t[Optional] Flip the assembly to put vertical slats on top. Default is horizontal on top.")
    print("")
    exit()



if __name__ == "__main__":
    try:
        # starts at the second element of argv since the first one is the script name
        # extraparams are extra arguments passed after all option/keywords are assigned
        # opts is a list containing the pair "option"/"value"
        opts, extraparams = getopt.getopt(sys.argv[1:], "hd:spx:l:n:f",
                            ["help", "dir=", "svg", "pdf", "xml=", "labels=", "name=", "flip"])
        #print 'Opts:',opts
        #print 'Extra parameters:',extraparams
    except:
        print("")
        print("Unrecognized option:" + str(sys.argv[1:]))
        DisplayHelp()

    if len(opts) == 0:
        DisplayHelp()

    b_pdf = False
    b_svg = False
    directory = None
    base_file_name = None
    pdf_file_name = None
    svg_file_name = None
    xml_file_name = None
    b_show_labels = True
    b_flip_assembly = False

    for o,p in opts:
        if o in ['-h', '--help']:
            DisplayHelp()
        elif o in ['-n', '--name']:
            base_file_name = p
        elif o in ['-p', '--pdf']:
            b_pdf = True
        elif o in ['-s', '--svg']:
            b_svg = True
        elif o in ['-x', '--xml']:
            xml_file_name = p
        elif o in ['-d', '--dir']:
            directory = p
        elif o in ['-l', '--labels']:
            if (p == 'F') or (p == 'False'):
                b_show_labels = False
        elif o in ['-f', '--flip']:
            b_flip_assembly = True
        else:
            DisplayHelp()

    if xml_file_name is None:
        print("No input file name given.")
        DisplayHelp()

    xml_file_names = glob.glob(xml_file_name)
    for xml_file_name in xml_file_names:

        curr_base_file_name = base_file_name
        if curr_base_file_name is None:
            curr_base_file_name = os.path.basename(xml_file_name[0:-4])
            
        if directory is None:
            xml_file_name = os.path.abspath(xml_file_name)
            directory = os.path.dirname(xml_file_name)

        if not os.path.exists(directory):
            os.makedirs(directory)
            
        if b_svg == True:
            svg_file_name = f'{directory}/{curr_base_file_name}.svg'

        if b_pdf == True:
            pdf_file_name = f'{directory}/{curr_base_file_name}.pdf'

        slat_system = SlatSystem()
        slat_system.ReadFromFile(xml_file_name)

        display_mask = DISPLAY_ALL if b_show_labels else DISPLAY_NO_GLUES

        slat_system.assembly.WriteToPDForSVG(pdfFileName=pdf_file_name, svgFileName=svg_file_name, displayMask=display_mask, sideLen=50, b_flip_assembly=b_flip_assembly)

        
