from PyTASTileTypeLight import *
import math
import sys
import getopt
import copy
import os


def DisplayHelp():
    print("\nThis script takes as input a binary string, b, for the input to a zig-zag TM and possibly the names")
    print("of output files, and outputs files for an aTAM system which is a zig-zag system that simulates the TM.")
    print("\nUsage:")
    print("-h, --help\t\t\t\tDisplays this help message.")
    print("-b <bin>, --bin=<bin>\t\t\tBinary string as input.")
    print("-s <tdp file>, --seed=<tdp file>\t[Optional] TDP file to output. If not specified,")
    print("\t\t\t\t\t`zig-zag_TM_input<bin>.tdp' will be used for given base value.")
    print("-t <tds file>, --tileset=<tds file>\t[Optional] TDS file to output. If not specified,")
    print("\t\t\t\t\t`zig-zag_TM_input<bin>.tds' will be used for given base value.")
    print("-d <dir>, --dir=<dir>\t\t\t[Optional] Directory for output files. If not specified,")
    print("\t\t\t\t\tthe current directory will be used.")
    print("-f <txt file>, --input=<txt file>\t[Optional] Text file containing the description of")
    print("\t\t\t\t\ta Turing machine following the format of the example 'turing-machine.txt'.")
    print("-o, --optim\t\t\t\t[Optional] Optimize zig-zag behavior by avoiding no-ops.")
    print("-u, --ultra\t\t\t\t[Optional] Ultra optimization of zig-zag behavior.")
    print("-i, --io\t\t\t\t[Optional] If this option is specified, I/O markings will be added to")
    print("\t\t\t\t\t glue labels.")
    print("")
    exit()

# Variables to hold I/O markings if they are to be used.
iN = ''
iE = ''
iS = ''
iW = ''
oN = ''
oE = ''
oS = ''
oW = ''

tape_alphabet = []
states = []
start_state = ''
transitions = []

def GetTMDefinitions(file_name):

    file = open(file_name, 'r')
    lines = file.readlines()

    p = 0


    for line in lines:
        if(len(line) > 0):
            if line.strip() == 'Alphabet:':
                p = 0
            elif line.strip() == 'States:':
                p = 1
            elif line.strip() == 'Start State:':
                p = 2
            elif line.strip() == 'Transitions:':
                p = 3
            else:
                if(p == 0):
                    global tape_alphabet
                    tape_alphabet = line.strip().split(',')
                elif(p == 1):
                    global states
                    states = line.strip().split(',')
                elif(p == 2):
                    global start_state
                    start_state = line.strip()
                elif(p == 3):
                    global transitions
                    t_list = line.strip().split(',')
                    transitions.append(tuple(t_list))

def SetStrengthOneGlues(tile: PyTASTileType):
    
    tile.sides[N].strength = 1
    tile.sides[E].strength = 1
    tile.sides[S].strength = 1
    tile.sides[W].strength = 1

def AddTile(tiles: list, tile: PyTASTileType):

    if not tile in tiles:
        tiles.append(tile)

def GetUltraCompactTransTiles(transitions):

    tile_types = []

    for transition in transitions:
        
        start_state = transition[0]
        start_symbol = transition[1]
        end_state = transition[2]
        end_symbol = transition[3]
        direction = transition[4]

        # === Odd rows (left-to-right growth) ===
        if direction == 'L':
            # Actual head tile
            t = PyTASTileType()
            t.name = f'ODD-{start_state},{start_symbol}'
            t.label = f'{start_state},{start_symbol}>'
            t.sides[N].label = f'{oN}{start_state},{start_symbol}'
            t.sides[E].label = f'{oE}>'
            t.sides[S].label = f'{iS}{start_symbol}'
            t.sides[W].label = f'{iW}{start_state}'
            t.color = 'Red'
            SetStrengthOneGlues(t)
            AddTile(tile_types, t)

            # Signal passer
            t = PyTASTileType()
            t.name = f'ODD-to-EVEN-{start_state},{start_symbol}'
            t.label = f'{end_symbol}'
            t.sides[N].label = f'{oN}{end_symbol}'
            t.sides[E].label = f'{iE}<'
            t.sides[S].label = f'{iS}{start_state},{start_symbol}'
            t.sides[W].label = f'{oW}{end_state}'
            SetStrengthOneGlues(t)
            AddTile(tile_types, t)

        else:

            t = PyTASTileType()
            t.name = f'ODD-{start_state},{start_symbol}'
            t.label = f'{start_state},{start_symbol}>'
            t.sides[N].label = f'{oN}{end_symbol}'
            t.sides[E].label = f'{oE}{end_state}'
            t.sides[S].label = f'{iS}{start_symbol}'
            t.sides[W].label = f'{iW}{start_state}'
            t.color = 'Red'
            SetStrengthOneGlues(t)
            AddTile(tile_types, t)

        # === Even rows (right-to-left growth) ===
        if direction == 'L':

            t = PyTASTileType()
            t.name = f'EVEN-{start_state},{start_symbol}'
            t.label = f'<{start_state},{start_symbol}'
            t.sides[N].label = f'{oN}{end_symbol}'
            t.sides[E].label = f'{iE}{start_state}'
            t.sides[S].label = f'{iS}{start_symbol}'
            t.sides[W].label = f'{oW}{end_state}'
            t.color = 'Red'
            SetStrengthOneGlues(t)
            AddTile(tile_types, t)

        else:
            # Actual head tile
            t = PyTASTileType()
            t.name = f'EVEN-{start_state},{start_symbol}'
            t.label = f'<{start_state},{start_symbol}'
            t.sides[N].label = f'{oN}{start_state},{start_symbol}'
            t.sides[E].label = f'{iE}{start_state}'
            t.sides[S].label = f'{iS}{start_symbol}'
            t.sides[W].label = f'{oW}<'
            t.color = 'Red'
            SetStrengthOneGlues(t)
            AddTile(tile_types, t)

            # Signal passer tile
            t = PyTASTileType()
            t.name = f'EVEN-to-ODD-{start_state},{start_symbol}'
            t.label = f'{end_symbol}'
            t.sides[N].label = f'{oN}{end_symbol}'
            t.sides[E].label = f'{oE}{end_state}'
            t.sides[S].label = f'{iS}{start_state},{start_symbol}'
            t.sides[W].label = f'{iW}>'
            SetStrengthOneGlues(t)
            AddTile(tile_types, t)

    return tile_types

def GetCompactTransitionTiles(transitions, tape_alphabet):

    tile_types = []

    for transition in transitions:
        
        start_state = transition[0]
        start_symbol = transition[1]
        end_state = transition[2]
        end_symbol = transition[3]
        direction = transition[4]

        # ===== Step 1: "Head" tiles =====
            
        # === Odd row & signal from left ===
        t = PyTASTileType()
        t.name = f'ODD-W-{start_state},{start_symbol}'
        t.label = f'{start_state},{start_symbol}>'
        t.sides[S].label = f'{iS}{start_symbol}'
        t.sides[W].label = f'{iW}{start_state}'
        t.color = 'Red'

        if direction == 'L':
            t.sides[N].label = f'{oN}{start_state}*,{start_symbol}'
            t.sides[E].label = f'{oE}>'

        else:
            t.sides[N].label = f'{oN}{end_symbol}'
            t.sides[E].label = f'{oE}{start_state}*,{start_symbol}'
        
        SetStrengthOneGlues(t)
        AddTile(tile_types, t)

        # === Odd row & signal from below ===
        t = PyTASTileType()
        t.name = f'ODD-S-{start_state},{start_symbol}'
        t.label = f'{start_state},{start_symbol}>'
        t.sides[S].label = f'{iS}{start_state},{start_symbol}'
        t.sides[W].label = f'{iW}>'
        t.color = 'Red'

        if direction == 'L':
            t.sides[N].label = f'{oN}{start_state}*,{start_symbol}'
            t.sides[E].label = f'{oE}>'

        else:
            t.sides[N].label = f'{oN}{end_symbol}'
            t.sides[E].label = f'{oE}{start_state}*,{start_symbol}'

        SetStrengthOneGlues(t)
        AddTile(tile_types, t)
        
        # === Even row & signal from the right ===
        t = PyTASTileType()
        t.name = f'EVEN-E-{start_state},{start_symbol}'
        t.label = f'<{start_state},{start_symbol}'
        t.sides[E].label = f'{iE}{start_state}'
        t.sides[S].label = f'{iS}{start_symbol}'
        t.color = 'Red'
        
        if direction == 'L':
            t.sides[N].label = f'{oN}{end_symbol}'
            t.sides[W].label = f'{oW}{start_state}*,{start_symbol}'

        else:
            t.sides[N].label = f'{oN}{start_state}*,{start_symbol}'
            t.sides[W].label = f'{oW}<'

        SetStrengthOneGlues(t)
        AddTile(tile_types, t)

        # === Even row & signal from below ===
        t = PyTASTileType()
        t.name = f'EVEN-S-{start_state},{start_symbol}'
        t.label = f'<{start_state},{start_symbol}'
        t.sides[E].label = f'{iE}<'
        t.sides[S].label = f'{iS}{start_state},{start_symbol}'
        t.color = 'Red'

        if direction == 'L':
            t.sides[N].label = f'{oN}{end_symbol}'
            t.sides[W].label = f'{oW}{start_state}*,{start_symbol}'

        else:
            t.sides[N].label = f'{oN}{start_state}*,{start_symbol}'
            t.sides[W].label = f'{oW}<'

        SetStrengthOneGlues(t)
        AddTile(tile_types, t)

        # ===== Step 2: Odd row; tiles fill left to right =====

        # Transitions left: Pass signal north
        if direction == 'L':
            t = PyTASTileType()
            t.name = f'ODD-trans-{start_state},{start_symbol}'
            t.label = f'{end_symbol}>'
            t.sides[N].label = f'{oN}{end_symbol}'
            t.sides[E].label = f'{iE}<'
            t.sides[S].label = f'{iS}{start_state}*,{start_symbol}'
            t.sides[W].label = f'{oW}{end_state}'

            SetStrengthOneGlues(t)
            AddTile(tile_types, t)

        # Else transitions right: Pass signal east
        else:
            for symbol in tape_alphabet:
                t = PyTASTileType()
                t.name = f'ODD-trans-{start_state},{start_symbol};{symbol}'
                t.label = f'{symbol}'
                t.sides[N].label = f'{oN}{end_state},{symbol}'
                t.sides[E].label = f'{oE}>'
                t.sides[S].label = f'{iS}{symbol}'
                t.sides[W].label = f'{iW}{start_state}*,{start_symbol}'

                SetStrengthOneGlues(t)
                AddTile(tile_types, t)

        # ===== Step 3: Even row; tiles fill right to left =====

        # Transitions left: Pass signal west
        if direction == 'L':
            for symbol in tape_alphabet:
                t = PyTASTileType()
                t.name = f'EVEN-trans-{start_state},{start_symbol};{symbol}'
                t.label = f'{symbol}'
                t.sides[N].label = f'{oN}{end_state},{symbol}'
                t.sides[E].label = f'{iE}{start_state}*,{start_symbol}'
                t.sides[S].label = f'{iS}{symbol}'
                t.sides[W].label = f'{oW}<'

                SetStrengthOneGlues(t)
                AddTile(tile_types, t)

        # Else transitions right: Pass signal north
        else:
            t = PyTASTileType()
            t.name = f'EVEN-trans-{start_state},{start_symbol}'
            t.label = f'<{end_symbol}'
            t.sides[N].label = f'{oN}{end_symbol}'
            t.sides[E].label = f'{oE}{end_state}'
            t.sides[S].label = f'{iS}{start_state}*,{start_symbol}'
            t.sides[W].label = f'{iW}>'

            SetStrengthOneGlues(t)
            AddTile(tile_types, t)

    return tile_types

def GetTransitionTiles(transitions, tape_alphabet):
    tile_types = []

    for transition in transitions:
        start_state = transition[0]
        start_symbol = transition[1]
        end_state = transition[2]
        end_symbol = transition[3]
        direction = transition[4]

        # Odd rows (left-to-right growth): right transitions executed, left transitions ignored
        if direction == 'L':
            t = PyTASTileType()
            t.name = f'ODD-{start_state},{start_symbol}'
            t.label = f'{start_state},{start_symbol}>'
            t.sides[N].label = f'{oN}{start_state},{start_symbol}'
            t.sides[N].strength = 1
            t.sides[S].label = f'{iS}{start_state},{start_symbol}'
            t.sides[S].strength = 1
            t.sides[E].label = f'{oE}>'
            t.sides[E].strength = 1
            t.sides[W].label = f'{iW}>'
            t.sides[W].strength = 1
            t.color = 'Pink'
            tile_types.append(t)
        else:
            # Execute transition and pass new state to the right
            t = PyTASTileType()
            t.name = f'ODD-{start_state},{start_symbol}'
            t.label = f'{end_symbol}>'
            t.sides[N].label = f'{oN}{end_symbol}'
            t.sides[N].strength = 1
            t.sides[S].label = f'{iS}{start_state},{start_symbol}'
            t.sides[S].strength = 1
            t.sides[E].label = f'{oE}{end_state}>'
            t.sides[E].strength = 1
            t.sides[W].label = f'{iW}>'
            t.sides[W].strength = 1
            tile_types.append(t)

            for symbol in tape_alphabet:
                # Get the new state from the left
                t = PyTASTileType()
                t.name = f'ODD-{end_state},{symbol}!'
                t.label = f'{end_state},{symbol}>'
                t.sides[N].label = f'{oN}{end_state},{symbol}'
                t.sides[N].strength = 1
                t.sides[S].label = f'{iS}{symbol}'
                t.sides[S].strength = 1
                t.sides[E].label = f'{oE}>'
                t.sides[E].strength = 1
                t.sides[W].label = f'{iW}{end_state}>'
                t.sides[W].strength = 1
                t.color = 'Red'
                if not t in tile_types:
                    tile_types.append(t)
                #else:
                    #print('found duplicate tile type')

        # Even rows (right-to-left growth): left transitions executed, right transitions ignored
        if direction == 'L':
            t = PyTASTileType()
            t.name = f'EVEN-{start_state},{start_symbol}'
            t.label = f'<{end_symbol}'
            t.sides[N].label = f'{oN}{end_symbol}'
            t.sides[N].strength = 1
            t.sides[S].label = f'{iS}{start_state},{start_symbol}'
            t.sides[S].strength = 1
            t.sides[E].label = f'{iE}<'
            t.sides[E].strength = 1
            t.sides[W].label = f'{oW}<{end_state}'
            t.sides[W].strength = 1
            tile_types.append(t)
            
            for symbol in tape_alphabet:
                # Get the new state from the left
                t = PyTASTileType()
                t.name = f'EVEN-{end_state},{symbol}!'
                t.label = f'<{end_state},{symbol}'
                t.sides[N].label = f'{oN}{end_state},{symbol}'
                t.sides[N].strength = 1
                t.sides[S].label = f'{iS}{symbol}'
                t.sides[S].strength = 1
                t.sides[E].label = f'{iE}<{end_state}'
                t.sides[E].strength = 1
                t.sides[W].label = f'{oW}<'
                t.sides[W].strength = 1
                t.color = 'Red'
                if not t in tile_types:
                    tile_types.append(t)
                #else:
                    #print('found duplicate tile type')
        else:
            t = PyTASTileType()
            t.name = f'EVEN-{start_state},{start_symbol}'
            t.label = f'<{start_state},{start_symbol}'
            t.sides[N].label = f'{oN}{start_state},{start_symbol}'
            t.sides[N].strength = 1
            t.sides[S].label = f'{iS}{start_state},{start_symbol}'
            t.sides[S].strength = 1
            t.sides[E].label = f'{iE}<'
            t.sides[E].strength = 1
            t.sides[W].label = f'{oW}<'
            t.sides[W].strength = 1
            t.color = 'Pink'
            tile_types.append(t)
                
    return tile_types


def GetSymbolCopyTiles(tape_alphabet):
    tile_types = []

    for symbol in tape_alphabet:

        #################
        # Odd row copy tiles
        t = PyTASTileType()
        t.name = f'ODD-COPY-{symbol}'
        t.label = f'{symbol}>'
        t.sides[N].label = f'{oN}{symbol}'
        t.sides[N].strength = 1
        t.sides[S].label = f'{iS}{symbol}'
        t.sides[S].strength = 1
        t.sides[E].label = f'{oE}>'
        t.sides[E].strength = 1
        t.sides[W].label = f'{iW}>'
        t.sides[W].strength = 1
        tile_types.append(t)

        #################
        # Even row copy tiles
        t = PyTASTileType()
        t.name = f'EVEN-COPY-{symbol}'
        t.label = f'<{symbol}'
        t.sides[N].label = f'{oN}{symbol}'
        t.sides[N].strength = 1
        t.sides[S].label = f'{iS}{symbol}'
        t.sides[S].strength = 1
        t.sides[E].label = f'{iE}<'
        t.sides[E].strength = 1
        t.sides[W].label = f'{oW}<'
        t.sides[W].strength = 1
        tile_types.append(t) 

    return tile_types


def GetInputTileTypes(bin_input, start_state, optimize=0):
    tile_types = []

    bin_input = f'{bin_input}_'

    for i in range(len(bin_input)+1):
        b = bin_input[i] if i < len(bin_input) else None
        
        #################
        # Input bit tile
        t = PyTASTileType()
        t.name = f'In{i}'
        t.label = str(b)
        if i == 0:
            t.sides[W].label = f'{oW}e-w_'
            t.sides[W].strength = 2
            if optimize == 1:
                t.sides[N].label = f'{oN}{start_state}*,{b}'
            else:
                t.sides[N].label = f'{oN}{start_state},{b}'
            t.sides[N].strength = 1
            t.color = 'Red'
        else:
            t.sides[W].label = f'{oW}{i}'
            t.sides[W].strength = 2
            t.sides[N].label = f'{oN}{b}'
            t.sides[N].strength = 1

        if i == (len(bin_input) - 1):
            t.sides[E].label = f'{iE}e-eSEED'
            t.sides[E].strength = 2
        elif i == (len(bin_input)):
            t.name = 'SEED'
            t.sides[W].label = f'{oW}e-eSEED'
            t.sides[W].strength = 2
            t.sides[N].label = f'{oN}e-e'
            t.sides[N].strength = 1
        else:
            t.sides[E].label = f'{iE}{i+1}'
            t.sides[E].strength = 2

        tile_types.append(t)

    t = PyTASTileType()
    t.name = f'In_'
    t.label = '_'
    t.sides[E].label = f'{iE}e-w_'
    t.sides[E].strength = 2
    t.sides[N].label = f'{oN}_'
    t.sides[N].strength = 1
    t.sides[W].label = f'{oW}e-w'
    t.sides[W].strength = 2
    tile_types.append(t)

    return tile_types


def GetBoundaryTileTypes():
    tile_types = []

##    #################
##    # Seed tile
##    t = PyTASTileType()
##    t.name = 'SEED'
##    t.label = 'SEED'
##    t.sides[W].label = 'e-w'
##    t.sides[W].strength = 2
##    t.sides[E].label = 'e-e'
##    t.sides[E].strength = 2
##    tile_types.append(t)

    #################
    # Even row helper tiles
    t = PyTASTileType()
    t.name = 'EVEN-H-WEST'
    t.label = 'EHW'
    t.sides[N].label = f'{oN}e-w'
    t.sides[N].strength = 2
    t.sides[E].label = f'{iE}e-w'
    t.sides[E].strength = 2
    t.color = 'Gray'
    tile_types.append(t)

    # t = PyTASTileType()
    # t.name = 'EVEN-H-EAST'
    # t.label = 'EHE'
    # t.sides[W].label = 'e-e'
    # t.sides[W].strength = 2
    # t.sides[N].label = 'e-e'
    # t.sides[N].strength = 1
    # t.color = 'Gray'
    # tile_types.append(t)

    #################
    # Odd row helper tiles
    # t = PyTASTileType()
    # t.name = 'ODD-H-WEST'
    # t.label = 'OHW'
    # t.sides[N].label = 'o-e'
    # t.sides[N].strength = 1
    # t.sides[E].label = 'o-w'
    # t.sides[E].strength = 2
    # t.color = 'Gray'
    # tile_types.append(t)

    t = PyTASTileType()
    t.name = 'ODD-H-EAST'
    t.label = 'OHE'
    t.sides[W].label = f'{iW}o-e'
    t.sides[W].strength = 2
    t.sides[N].label = f'{oN}o-w'
    t.sides[N].strength = 2
    t.color = 'Gray'
    tile_types.append(t)

    #################
    # Odd row interior tiles
    t = PyTASTileType()
    t.name = 'ODD-I-WEST'
    t.label = 'OIW'
    t.sides[S].label = f'{iS}e-w'
    t.sides[S].strength = 2
    #t.sides[W].label = 'o-w'
    #t.sides[W].strength = 2
    t.sides[E].label = f'{oE}>'
    t.sides[E].strength = 1
    t.sides[N].label = f'{oN}o-e'#'_'
    t.sides[N].strength = 1
    t.color = 'Gray'
    tile_types.append(t)

    t = PyTASTileType()
    t.name = 'ODD-I-EAST'
    t.label = 'OIE'
    t.sides[S].label = f'{iS}e-e'
    t.sides[S].strength = 1
    t.sides[E].label = f'{oE}o-e'
    t.sides[E].strength = 2
    t.sides[N].label = f'{oN}_'
    t.sides[N].strength = 1
    t.sides[W].label = f'{iW}>'
    t.sides[W].strength = 1
    t.color = 'Gray'
    tile_types.append(t)

    #################
    # Even row interior tiles
    t = PyTASTileType()
    t.name = 'EVEN-I-WEST'
    t.label = 'EIW'
    t.sides[S].label = f'{iS}o-e'
    t.sides[S].strength = 1
    t.sides[W].label = f'{oW}e-w'
    t.sides[W].strength = 2
    t.sides[N].label = f'{oN}_'
    t.sides[N].strength = 1
    t.sides[E].label = f'{iE}<'
    t.sides[E].strength = 1
    t.color = 'Gray'
    tile_types.append(t)

    t = PyTASTileType()
    t.name = 'EVEN-I-EAST'
    t.label = 'EIE'
    t.sides[S].label = f'{iS}o-w'
    t.sides[S].strength = 2
    #t.sides[E].label = 'e-e'
    #t.sides[E].strength = 2
    t.sides[W].label = f'{oW}<'
    t.sides[W].strength = 1
    t.sides[N].label = f'{oN}e-e'#'_'
    t.sides[N].strength = 1
    t.color = 'Gray'
    tile_types.append(t)


    return tile_types



if __name__ == "__main__":
    try:
        # starts at the second element of argv since the first one is the script name
        # extraparams are extra arguments passed after all option/keywords are assigned
        # opts is a list containing the pair "option"/"value"
        opts, extraparams = getopt.getopt(sys.argv[1:], "hb:s:t:d:f:oui",
                            ["help", "bin=", "seed=", "tileset=", "dir=",  "input=", "optim", "ultra", "io"])
        #print 'Opts:',opts
        #print 'Extra parameters:',extraparams
    except:
        print("")
        print("Unrecognized option:" + str(sys.argv[1:]))
        DisplayHelp()

    if len(opts) == 0:
        DisplayHelp()

    bin_input = None
    tdp_file = None
    tds_file = None
    directory = '.'
    optimize = 0
    input_file = None
    opt_str = ''
    b_add_io = False

    for o,p in opts:
        if o in ['-h', '--help']:
            DisplayHelp()
        elif o in ['-b', '--bin']:
            bin_input = p
        elif o in ['-s', '--seed']:
            tdp_file = p
        elif o in ['-t', '--tileset']:
            tds_file = p
        elif o in ['-d', '--dir']:
            directory = p
        elif o in ['-o', '--optim']:
            optimize = 1
            opt_str += '_optim'
        elif o in ['-u', '--ultra']:
            optimize = 2
            opt_str += '_ultra'
        elif o in ['-f', '--input']:
            input_file = p
        elif o in ['-i', '--io']:
            b_add_io = True
        else:
            DisplayHelp()

    if bin_input is None:
        print("\nNo binary input string specified!")
        DisplayHelp()
    else:
        for b in bin_input:
            if not b in ['0','1']:
                print("\nInput string must be binary!")
                DisplayHelp()

    io_add_str = ''
    if b_add_io:
        iN = 'v'
        iE = '<'
        iS = '^'
        iW = '>'
        oN = '^'
        oE = '>'
        oS = 'v'
        oW = '<'
        io_add_str = '_IO'

    if tdp_file is None:
        tdp_file = f'zig-zag_TM_input{bin_input}{opt_str}{io_add_str}.tdp'
    if tds_file is None:
        tds_file = f'zig-zag_TM_input{bin_input}{opt_str}{io_add_str}.tds'
    if input_file is None:
        input_file = 'turing-machine.txt'

    #imports Turing Machine definition from file specified
    GetTMDefinitions(input_file)

    #counter_tiles, (lsb_tiles, interior_tiles, msb_tiles) = GetCounterTileTypes(base)
    #all_tiles = counter_tiles + lsb_tiles + interior_tiles + msb_tiles
    bin_input_tiles = GetInputTileTypes(bin_input, start_state, optimize)
    boundary_tiles = GetBoundaryTileTypes()
    copy_tiles = GetSymbolCopyTiles(tape_alphabet)
    transition_tiles = None
    if optimize == 0:
        transition_tiles = GetTransitionTiles(transitions, tape_alphabet)
    elif optimize == 1:
        transition_tiles = GetCompactTransitionTiles(transitions, tape_alphabet)
    else:
        transition_tiles = GetUltraCompactTransTiles(transitions)
    
    all_tiles = bin_input_tiles + boundary_tiles + copy_tiles + transition_tiles
    print(f'Number of zig-zag TM tiles is {len(all_tiles)}.')

    #new_lsb_tiles, new_interior_tiles, new_msb_tiles = apply_function(base, lsb_tiles, interior_tiles, msb_tiles)
    #all_tiles = counter_tiles + new_lsb_tiles + new_interior_tiles + new_msb_tiles
    #print(f'Number of counter tiles after applying function is {len(all_tiles)}.')
    tile_set = all_tiles

    if not os.path.exists(directory):
        os.makedirs(directory)

    with open(f'{directory}/{tdp_file}', 'w') as f:
        f.write(f'{tds_file}\n')
        f.write('Temperature=2\n')
        f.write(f'SEED 0 0\n')

    with open(f'{directory}/{tds_file}', 'w') as f:
        for tile in tile_set:
            f.write(f'{tile.Output()}\n')
    print('Tile system output complete.')


